from pathlib import Path

DIR_PATH = Path(__file__).resolve().parent.parent
API_URL = "http://127.0.0.1:8765"
DECK_NAME = "korean"
MODEL_NAME = "Basic (裏表反転カード付き)+sentense"
MP3_PATH = DIR_PATH / "data"
